﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace Praktikum2
{
    class Material
    {
        Brush brush;
        double lambda;
        string name;

        public Brush Brush
        { get { return brush; } }

        public double Lambda
        { get { return lambda; } }

        public Material (string name)
        {
            this.name = name;
            switch(name)
            {
                case "PUR":
                    brush = new SolidColorBrush(Colors.DarkMagenta);
                    lambda = 0.28;
                    break;
                case "Beton":
                    brush = new RadialGradientBrush(Colors.Azure, Colors.DarkGreen);
                    lambda = 2.1;
                    break;
                case "Holz":
                    brush = new LinearGradientBrush(Colors.Red, Colors.SandyBrown, 45.0);
                    lambda = 0.15;
                    break;
                default:
                    throw new ApplicationException("Kein Material dieses Namens verfügbar!");
            }
        }

        // alternative Lösung statt des Konstruktors oben
        Material(double lambda, Brush brush, string name) // privater (!) Konstruktor
        {
            this.lambda = lambda;
            this.brush = brush;
            this.name = name;
        }
        public static readonly Material PUR = new Material(0.28, new SolidColorBrush(Colors.DarkMagenta), "PUR");
        public static readonly Material Beton = new Material(2.1, new RadialGradientBrush(Colors.Azure, Colors.DarkGreen), "Beton");
        public static readonly Material Holz = new Material(0.15, new LinearGradientBrush(Colors.Red, Colors.SandyBrown, 45.0), "Holz");
        // Ende alternative Lösung

        public override string ToString()
        {
            return name + " " + lambda;
        }
    }

    abstract class Schicht
    {
        public abstract double Dicke
        {
            get;
        }

        public abstract double BestimmeR();

        public double BestimmeU()
        {
            return 1.0 / ( BestimmeR() + 0.17 );
        }

        public abstract void Zeichne(Canvas canvas, double links, double oben, double rechts, double unten);
    }

    class Einfachschicht : Schicht
    {
        double dicke;
        Material material;

        public Einfachschicht(Material material, double dicke)
        {
            this.material = material;
            this.dicke = dicke;
        }

        public override double BestimmeR()
        {
            return Dicke / material.Lambda;
        }

        public override void Zeichne(Canvas canvas, double links, double oben, double rechts, double unten)
        {
            if (links > rechts || oben > unten)
            {
                return;
            }

            Rectangle r = new Rectangle();
            r.Width = rechts - links;
            r.Height = unten - oben;
            r.Fill = material.Brush;
            canvas.Children.Add(r);
            Canvas.SetLeft(r, links);
            Canvas.SetTop(r, oben);
        }

        public override double Dicke
        {
            get { return dicke; }
        }

        public override string ToString()
        {
            return material.ToString() + " " + dicke;
        }
    }

    class Mehrfachschicht : Schicht
    {
        List<Schicht> teilschichten = new List<Schicht>();

        public void FügeSchichtHinzu(Schicht schicht)
        {
            teilschichten.Add(schicht);
        }

        public override double BestimmeR()
        {
            return teilschichten.Sum(s => s.BestimmeR());
        }

        public override void Zeichne(Canvas canvas, double links, double oben, double rechts, double unten)
        {
            if(rechts - links <= 8.0)
            { 
                return;
            }

            double gesamtdicke = teilschichten.Sum(s => s.Dicke);
            double x = 4.0;
            foreach (var s in teilschichten)
            {
                double breite = s.Dicke * (rechts - links - 8.0) / gesamtdicke;
                s.Zeichne(canvas, links + x, oben + 4.0, links + x + breite, unten - 4.0);
                x += breite;
            }
        }
        
        public override double Dicke
        {
            get { return teilschichten.Sum(s => s.Dicke); }
        }

        public override string ToString()
        {
            return "Mehrfachschicht aus " + teilschichten.Count + " Schichten";
        }
    }
}